@@ -1,8 +1,8 @@
-# 1 "/home/micael/devel/libxc/svn/tags/2.0.3/src/libxc_master.F90"
+# 1 "/home/micael/devel/libxc/svn/tags/2.1.0/src/libxc_master.F90"
 # 1 "<command-line>"
 # 1 "/usr/include/stdc-predef.h" 1 3 4
 # 1 "<command-line>" 2
-# 1 "/home/micael/devel/libxc/svn/tags/2.0.3/src/libxc_master.F90"
+# 1 "/home/micael/devel/libxc/svn/tags/2.1.0/src/libxc_master.F90"
 !! Copyright (C) 2003-2006 M. Marques, A. Castro, A. Rubio, G. Bertsch
 !!
 !! This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 !! 02110-1301, USA.
 !!
 !! $Id: libxc.f90 3550 2007-11-19 14:32:49Z marques $
-# 27 "/home/micael/devel/libxc/svn/tags/2.0.3/src/libxc_master.F90"
+# 27 "/home/micael/devel/libxc/svn/tags/2.1.0/src/libxc_master.F90"
 !-------------------------------------------------------------------
 module xc_f90_types_m
 
@@ -57,7 +57,8 @@ module xc_f90_lib_m
     XC_FAMILY_MGGA = 4, &
     XC_FAMILY_LCA = 8, &
     XC_FAMILY_OEP = 16, &
-    XC_FAMILY_HYB_GGA = 32
+    XC_FAMILY_HYB_GGA = 32, &
+    XC_FAMILY_HYB_MGGA = 64
 
   integer, parameter :: &
     XC_UNPOLARIZED = 1, & ! Spin unpolarized
@@ -86,9 +87,10 @@ module xc_f90_lib_m
     XC_FLAGS_STABLE = 512, &
     XC_FLAGS_DEVELOPMENT = 1024
 
-  ! This value was redefined as XC_GGA_X_LB, we define it here to keep
-  ! compatibility.
+  ! These are old names keep for compatibility, and that should disappear soon
   integer, parameter :: XC_GGA_XC_LB = 160
+  integer, parameter :: XC_GGA_K_ABSR1 = 506
+  integer, parameter :: XC_GGA_K_ABSR2 = 507
 
   !----------------------------------------------------------------
   interface
@@ -133,6 +135,15 @@ module xc_f90_lib_m
       character(len=*), intent(out) :: s ! the string that is output
     end subroutine xc_f90_info_refs
 
+    subroutine xc_f90_functional_get_name(func_number, func_string)
+      integer, intent(in) :: func_number
+      character(len=256), intent(out) :: func_string
+    end subroutine xc_f90_functional_get_name
+
+    integer function xc_f90_functional_get_number(func_string)
+      character(len=*), intent(in) :: func_string
+    end function xc_f90_functional_get_number
+
     integer function xc_f90_family_from_id(id)
       use xc_f90_types_m
       integer, intent(in) :: id
@@ -253,7 +264,8 @@ module xc_f90_lib_m
   ! GGAs
   !----------------------------------------------------------------
   interface
-    subroutine xc_f90_gga(p, np, rho, sigma, zk, vrho, vsigma, v2rho2, v2rhosigma, v2sigma2)
+    subroutine xc_f90_gga(p, np, rho, sigma, zk, vrho, vsigma, &
+        v2rho2, v2rhosigma, v2sigma2, v3rho3, v3rho2sigma, v3rhosigma2, v3sigma3)
       use xc_f90_types_m
       type(xc_f90_pointer_t), intent(in) :: p
       integer, intent(in) :: np
@@ -265,6 +277,10 @@ module xc_f90_lib_m
       real(xc_f90_kind), intent(out) :: v2rho2
       real(xc_f90_kind), intent(out) :: v2rhosigma
       real(xc_f90_kind), intent(out) :: v2sigma2
+      real(xc_f90_kind), intent(out) :: v3rho3
+      real(xc_f90_kind), intent(out) :: v3rho2sigma
+      real(xc_f90_kind), intent(out) :: v3rhosigma2
+      real(xc_f90_kind), intent(out) :: v3sigma3
     end subroutine xc_f90_gga
 
     subroutine xc_f90_gga_exc(p, np, rho, sigma, zk)
@@ -307,6 +323,18 @@ module xc_f90_lib_m
       real(xc_f90_kind), intent(out) :: v2rhosigma
       real(xc_f90_kind), intent(out) :: v2sigma2
     end subroutine xc_f90_gga_fxc
+
+    subroutine xc_f90_gga_kxc(p, np, rho, sigma, v3rho3, v3rho2sigma, v3rhosigma2, v3sigma3)
+      use xc_f90_types_m
+      type(xc_f90_pointer_t), intent(in) :: p
+      integer, intent(in) :: np
+      real(xc_f90_kind), intent(in) :: rho
+      real(xc_f90_kind), intent(in) :: sigma
+      real(xc_f90_kind), intent(out) :: v3rho3
+      real(xc_f90_kind), intent(out) :: v3rho2sigma
+      real(xc_f90_kind), intent(out) :: v3rhosigma2
+      real(xc_f90_kind), intent(out) :: v3sigma3
+    end subroutine xc_f90_gga_kxc
   end interface
 
   !----------------------------------------------------------------
@@ -336,16 +364,6 @@ module xc_f90_lib_m
   end interface
 
 
-  !----------------------------------------------------------------
-  interface
-    subroutine xc_f90_hyb_exx_coef(p, coef)
-      use xc_f90_types_m
-      type(xc_f90_pointer_t), intent(in) :: p
-      real(xc_f90_kind), intent(out) :: coef
-    end subroutine xc_f90_hyb_exx_coef
-  end interface
-
-
   !----------------------------------------------------------------
   interface
     subroutine xc_f90_gga_x_wpbeh_set_par(p, omega)
@@ -366,11 +384,43 @@ module xc_f90_lib_m
 
   !----------------------------------------------------------------
   interface
-    subroutine xc_f90_hyb_gga_xc_hse_set_par(p, omega)
+    subroutine xc_f90_gga_ak13_get_asymptotic(homo, asymp)
+      use xc_f90_types_m
+      real(xc_f90_kind), intent(in) :: homo
+      real(xc_f90_kind), intent(out) :: asymp
+    end subroutine xc_f90_gga_ak13_get_asymptotic
+  end interface
+
+  !----------------------------------------------------------------
+  interface
+    subroutine xc_f90_hyb_exx_coef(p, coef)
       use xc_f90_types_m
       type(xc_f90_pointer_t), intent(in) :: p
+      real(xc_f90_kind), intent(out) :: coef
+    end subroutine xc_f90_hyb_exx_coef
+
+    subroutine xc_f90_hyb_cam_coef(p, omega, alpha, beta)
+      use xc_f90_types_m
+      type(xc_f90_pointer_t), intent(in) :: p
+      real(xc_f90_kind), intent(out) :: omega, alpha, beta
+    end subroutine xc_f90_hyb_cam_coef
+  end interface
+
+
+  !----------------------------------------------------------------
+  interface
+    subroutine xc_f90_hyb_gga_xc_hse_set_par(p, beta, omega)
+      use xc_f90_types_m
+      type(xc_f90_pointer_t), intent(in) :: p
+      real(xc_f90_kind), intent(in) :: beta ! mixing
       real(xc_f90_kind), intent(in) :: omega ! range separation
     end subroutine xc_f90_hyb_gga_xc_hse_set_par
+
+    subroutine xc_f90_hyb_gga_xc_pbeh_set_par(p, alpha)
+      use xc_f90_types_m
+      type(xc_f90_pointer_t), intent(in) :: p
+      real(xc_f90_kind), intent(in) :: alpha ! mixing
+    end subroutine xc_f90_hyb_gga_xc_pbeh_set_par
   end interface
 
 
